package io.zfunny.j2dlt.dlt645.msg;

import io.zfunny.j2dlt.dlt645.Dlt645;
import io.zfunny.j2dlt.dlt645.net.AbstractDlt645Listener;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Arrays;

public class ControlRequest extends Dlt645Request {

    private byte[] dataIdentity;
    private byte[] password;
    private byte[] operator;
    private byte[] data;

    public ControlRequest() {
        super();

        setFunctionCode(Dlt645.CONTROL_ACTION);
        setDataLength(14);
    }

    public ControlRequest(byte[] dataIdentity, byte[] password, byte[] operator, byte[] data) {
        super();
        setFunctionCode(Dlt645.CONTROL_ACTION);
        setDataLength(4 + 4 + 4 + data.length);

        this.dataIdentity = Arrays.copyOf(dataIdentity, 4);
        this.password = Arrays.copyOf(password, 4);
        this.operator = Arrays.copyOf(operator, 4);
        this.data = Arrays.copyOf(data, data.length);
    }

    @Override
    public Dlt645Response getResponse() {
        return updateResponseWithHeader(new ControlResponse());
    }

    @Override
    public Dlt645Response createResponse(AbstractDlt645Listener listener) {
        return null; // TODO
    }

    public byte[] getDataIdentity() {
        byte[] dest = new byte[4];
        System.arraycopy(dataIdentity, 0, dest, 0, 4);
        return dest;
    }

    public void setDataIdentity(byte[] di) {
        this.dataIdentity = Arrays.copyOf(di, 4);
    }

    public byte[] getPassword() {
        byte[] dest = new byte[4];
        System.arraycopy(password, 0, dest, 0, 4);
        return dest;
    }

    public void setPassword(byte[] password) {

        this.password = Arrays.copyOf(password, 4);
    }

    public byte[] getOperator() {
        byte[] dest = new byte[4];
        System.arraycopy(operator, 0, dest, 0, 4);
        return dest;
    }

    public void setOperator(byte[] operator) {

        this.operator = Arrays.copyOf(operator, 4);
    }

    public byte[] getData() {
        byte[] dest = new byte[data.length];
        System.arraycopy(data, 0, dest, 0, data.length);
        return dest;
    }

    public void setData(byte[] data) {
        this.data = Arrays.copyOf(data, data.length);
    }

    @Override
    public void writeData(DataOutput dout) throws IOException {
        for (byte b : dataIdentity) {
            dout.writeByte(b + 0x33);
        }

        for (byte b : password) {
            dout.writeByte(b + 0x33);
        }

        for (byte b : operator) {
            dout.writeByte(b + 0x33);
        }

        for (byte b : data) {
            dout.writeByte(b + 0x33);
        }
    }

    @Override
    public void readData(DataInput din) throws IOException {
        din.readFully(dataIdentity, 0, 4);
        din.readFully(password, 0, 4);
        din.readFully(operator, 0, 4);
        // din.readFully(data, 0, 2); todo
    }

    @Override
    public byte[] getMessage() {
        byte[] result = new byte[12 + data.length];

        // FIXME +0x33 ???
        System.arraycopy(dataIdentity, 0, result, 0, 4);
        System.arraycopy(password, 0, result, 4, 4);
        System.arraycopy(operator, 0, result, 8, 4);
        System.arraycopy(data, 0, result, 12, data.length);

        return result;
    }
}
