package io.zfunny.j2dlt.dlt645.msg;

import io.zfunny.j2dlt.dlt645.Dlt645;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Arrays;

public class ReadAlarmResponse extends Dlt645Response {

    private int byteCount;
    private byte[] data;

    public ReadAlarmResponse() {
        super();
        setFunctionCode(Dlt645.READ_ALARM_RESPONSE);
    }

    public ReadAlarmResponse(byte[] data) {
        super();
        setFunctionCode(Dlt645.READ_ALARM_RESPONSE);
        setDataLength(data == null ? 0 : (data.length +1));

        this.data = data == null ?null : Arrays.copyOf(data, data.length);
        byteCount = data == null ? 0 : data.length;
    }

    public int getByCount() {
        return byteCount;
    }

    public synchronized byte[] getData() {
        byte[] dest = new byte[data.length];
        System.arraycopy(data, 0, dest, 0, dest.length);
        return dest;
    }

    public synchronized void setData(byte[] data) {
        byteCount = data == null ? 0 : data.length;
        this.data = data == null ? null : Arrays.copyOf(data, data.length);
        setDataLength(byteCount + 1);
    }

    @Override
    public void writeData(DataOutput dout) throws IOException {
        dout.writeByte(byteCount);
        for (byte b : data) {
            dout.writeByte( b + 0x33);
        }
    }

    @Override
    public void readData(DataInput din) throws IOException {
        byteCount = din.readUnsignedByte();

        data = new byte[byteCount];
        din.readFully(data, 0, data.length);

        for (int i = 0; i < data.length; i++) {
            data[i] = (byte) (data[i] - 0x33);
        }

        setDataLength(byteCount + 1);
    }

    @Override
    public byte[] getMessage() {
        byte[] result = new byte[byteCount + 1];
        result[0] = (byte)byteCount;

        System.arraycopy(data, 0, result, 1, byteCount);
        return result;
    }
}
