package io.zfunny.j2dlt.dlt645.msg;

import io.zfunny.j2dlt.dlt645.Dlt645;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Arrays;

public class ReadSubResponse extends Dlt645Response {

    private int byteCount;
    private byte[] data;
    private int seq;

    public ReadSubResponse() {
        super();
        setFunctionCode(Dlt645.READ_SUB_DATA_RESPONSE);
    }

    public ReadSubResponse(int functionCode) {
        super();
        setFunctionCode(functionCode);
    }

    public int getByteCount() {
        return byteCount;
    }

    public synchronized byte[] getData() {
        byte[] dest = new byte[data.length];
        System.arraycopy(data, 0, dest, 0, dest.length);
        return dest;
    }

    public synchronized void setData(byte[] data) {
        byteCount = data.length;
        this.data = Arrays.copyOf(data, data.length);
        setDataLength(byteCount + 2);
    }

    @Override
    public void writeData(DataOutput dout) throws IOException {
        dout.writeByte(byteCount);
        for (byte b : data) {
            dout.writeByte(b + 0x33);
        }
        dout.writeByte(seq);
    }

    @Override
    public void readData(DataInput din) throws IOException {
        byteCount = din.readUnsignedByte() - 5;
        byte[] dataIdentity = new byte[4];
        din.readFully(dataIdentity, 0, 4);
        for (int i = 0; i < dataIdentity.length; i++) {
            dataIdentity[i] = (byte) (dataIdentity[i] - 0x33);
        }

        data = new byte[byteCount];
        din.readFully(data, 0, data.length);
        for (int i = 0; i < data.length; i++) {
            data[i] = (byte)(data[i] - 0x33);
        }
        seq = din.readUnsignedByte();

        setDataLength(byteCount + 2);
    }

    @Override
    public byte[] getMessage() {
        byte[] result = new byte[byteCount + 2];
        result[0] = (byte)byteCount;
        System.arraycopy(data, 0, result, 1, byteCount);
        result[result.length - 1] = (byte)seq;

        return result;
    }
}
